pub trait AtomicStore: Primitive {
// Required method
unsafe fn atomic_store(
dst: *mut MaybeUninit<Self>,
val: MaybeUninit<Self>,
order: Ordering,
);
}Expand description
Atomic store.
This trait is sealed and cannot be implemented for types outside of atomic-maybe-uninit.
Required Methods
Sourceunsafe fn atomic_store(
dst: *mut MaybeUninit<Self>,
val: MaybeUninit<Self>,
order: Ordering,
)
unsafe fn atomic_store( dst: *mut MaybeUninit<Self>, val: MaybeUninit<Self>, order: Ordering, )
Stores a value into dst.
atomic_store takes an Ordering argument which describes the memory ordering of this operation.
Possible values are SeqCst, Release and Relaxed.
Safety
Behavior is undefined if any of the following conditions are violated:
dstmust be valid for writesdstmust be properly aligned to the size ofSelf. (For example, ifSelfisu128,dstmust be aligned to 16-byte even if the alignment ofu128is 8-byte.)ordermust beSeqCst,Release, orRelaxed.
The rules for the validity of the pointer follow the rules applied to
functions exposed by the standard library’s ptr module,
except that concurrent atomic operations on dst are allowed if the
pointer go through UnsafeCell::get.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.